
#include <dos.h>
#include <stdlib.h>

typedef unsigned char  BYTE;
typedef unsigned long  TIME;
typedef unsigned long  SYS_FLAGS;
typedef unsigned int   WORD;
typedef unsigned long  DWORD;

typedef signed   char      SBYTE;
typedef signed   short     SWORD;
typedef signed   long      SDWORD;
typedef signed   long      SQWORD; //NOrmalerweise 8 Byte Integer!!!
typedef unsigned long      QWORD;  //NOrmalerweise 8 Byte Integer!!!

#include "filesys.h"

//Exportable functions...
int fs_open(char* Filename, DWORD Mode, WORD *Result, char Attr, int lfn)
{
  int Segx, Offx, Retx, Resx, MdH, MdL;
  Segx = FP_SEG(Filename);
  Offx = FP_OFF(Filename);
  MdH = ((unsigned long)Mode) >> 16;
  MdL = Mode & 0xffff;
  lfn += 4;
  asm {
    mov ax, lfn
    mov es, Segx
    mov di, Offx
    mov bl, Attr   //Attribute
    mov bh, 0      //Aliashint
    mov dx, MdH
    mov cx, MdL
    int 0x30
    mov Retx, ax
    mov Resx, bx
  }
  if (Result != 0)
  *Result = Resx;
  return Retx;
}

int fs_close(int Handle)
{
  int Resx;
  asm {
    mov ax, 6
    mov si, Handle
    int 0x30
    mov Resx, ax
  }
  return Resx;
}

int fs_read(int Handle, char* Buffer, DWORD *Size)
{
  int Retx, Segx, Offx, SzL, SzH;
  Segx = FP_SEG(Buffer);
  Offx = FP_OFF(Buffer);
  SzL = (*Size) & 0xffff;
  SzH = (*Size) >> 16;
  asm {
    mov ax, 8
    mov si, Handle
    mov es, Segx
    mov di, Offx
    mov dx, SzH
    mov cx, SzL
    int 0x30
    mov SzH, dx
    mov SzL, cx
    mov Retx, ax
  }
  if (Size != 0)
    *Size = (((unsigned long) SzH) << 16) | (((unsigned long) SzL));
  return Retx;
}

int fs_write(int Handle, char* Buffer, DWORD *Size)
{
  int Retx, Segx, Offx, SzL, SzH;
  Segx = FP_SEG(Buffer);
  Offx = FP_OFF(Buffer);
  SzL = (*Size) & 0xffff;
  SzH = (*Size) >> 16;
  asm {
    mov ax, 9
    mov si, Handle
    mov es, Segx
    mov di, Offx
    mov dx, SzH
    mov cx, SzL
    int 0x30
    mov SzH, dx
    mov SzL, cx
    mov Retx, ax
  }
  if (Size != 0)
    *Size = (((unsigned long) SzH) << 16) | (((unsigned long) SzL));
  return Retx;
}

int fs_mkdir(char* Name, int lfn)
{
  int Segx, Offx, Retx;
  Segx = FP_SEG(Name);
  Offx = FP_OFF(Name);
  asm {
    mov ax, lfn
    mov es, Segx
    mov di, Offx
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

int fs_rmdir(char* Name, int lfn)
{
  int Segx, Offx, Retx;
  Segx = FP_SEG(Name);
  Offx = FP_OFF(Name);
  lfn += 2;
  asm {
    mov ax, lfn
    mov es, Segx
    mov di, Offx
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

int fs_unlink(char* Name, int lfn)
{
  int Segx, Offx, Retx;
  Segx = FP_SEG(Name);
  Offx = FP_OFF(Name);
  lfn += 10;
  asm {
    mov ax, lfn
    mov es, Segx
    mov di, Offx
    mov bx, 0x27 // Flags
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

// Documentation from seek.cpp:  (ad/031001)
/* Moves the file pointer (the target position for read and write  */
/* operations) for a file, according to the origin:                */
/*  0 - the file pointer is moved to the offset specified          */
/*  1 - the file pointer is moved relative to its current position */
/*  2 - the file pointer is moved from the end of the file (note   */
/*      that this Origin is not valid for directories, since the   */
/*      FileSize for a directory is always zero).                  */
/* On success, returns 0 and fills Offset                          */
/* with the new absolute target position.                          */
int fs_lseek(int Handle, long *Offset, int Origin)
{
  int Retx, OfL, OfH;
  OfH = *Offset >> 16;
  OfL = *Offset & 0xffff;
  asm {
    mov ax, 12
    mov si, Handle
    mov dx, OfH
    mov cx, OfL
    mov bx, Origin
    int 0x30
    mov Retx, ax
    mov OfH, dx
    mov OfL, cx
  }
  if (Offset != 0)
    *Offset = (((unsigned long) OfH) << 16) | ((unsigned long) OfL);
  return Retx;
}

#define fs_attrib fd32_dev_fs_attr_t
int fs_get_attributes(int Handle, fs_attrib *Attributes)
{
  int Retx, Segx, Offx;
  Segx = FP_SEG(Attributes);
  Offx = FP_OFF(Attributes);
  asm {
    mov ax, 13
    mov si, Handle
    mov es, Segx
    mov di, Offx
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

int fs_set_attributes(int Handle, fs_attrib *Attributes)
{
  int Retx, Segx, Offx;
  Segx = FP_SEG(Attributes);
  Offx = FP_OFF(Attributes);
  asm {
    mov ax, 14
    mov si, Handle
    mov es, Segx
    mov di, Offx
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

#define fs_dos_find fd32_fs_dosfind_t
int fs_dos_findfirst(char* Name, unsigned int attrib, fs_dos_find *sr)
{
  int Retx, Segx, Offx, Segx2, Offx2;
  Segx = FP_SEG(sr);
  Offx = FP_OFF(sr);
  Segx2 = FP_SEG(Name);
  Offx2 = FP_OFF(Name);
  asm {
    mov ax, 15
    mov es, Segx2
    mov di, Offx2
    mov dx, Segx
    mov cx, Offx
    mov bx, attrib
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

int fs_dos_findnext(fs_dos_find *sr)
{
  int Retx, Segx, Offx;
  Segx = FP_SEG(sr);
  Offx = FP_OFF(sr);
  asm {
    mov ax, 16
    mov dx, Segx
    mov cx, Offx
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

#define fs_lfn_find fd32_fs_lfnfind_t
int fs_lfn_findfirst(char* Name, int Attr, fs_lfn_find *sr)
{
  int Retx, Segx, Offx, Segx2, Offx2;
  Segx = FP_SEG(Name);
  Offx = FP_OFF(Name);
  Segx2 = FP_SEG(sr);
  Offx2 = FP_OFF(sr);
  asm {
    mov ax, 17
    mov es, Segx
    mov di, Offx
    mov dx, Segx2
    mov cx, Offx2
    mov bx, Attr
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

int fs_lfn_findnext(fs_lfn_find *sr)
{
  int Retx, Segx, Offx;
  Segx = FP_SEG(sr);
  Offx = FP_OFF(sr);
  asm {
    mov ax, 18
    mov dx, Segx
    mov cx, Offx
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

int fs_lfn_findclose(fs_lfn_find *sr)
{
  int Retx, Segx, Offx;
  Segx = FP_SEG(sr);
  Offx = FP_OFF(sr);
  asm {
    mov ax, 19
    mov dx, Segx
    mov cx, Offx
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

int fs_rename(char* OldName, char* NewName, int lfn)
{
  int Retx, Segx, Offx, Segx2, Offx2;
  Segx = FP_SEG(OldName);
  Offx = FP_OFF(OldName);
  Segx2 = FP_SEG(NewName);
  Offx2 = FP_OFF(NewName);
  lfn += 20;
  asm {
    mov ax, lfn
    mov es, Segx
    mov di, Offx
    mov dx, Segx2
    mov cx, Offx2
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

void drv_unmount(void)
{
  asm {
    mov ax, 22
    int 0x30
  }
}

int fs_setdefaultdrive(char drv)
{
  int Retx;
  asm {
    mov ax, 24
    mov bl, drv
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

char fs_getdefaultdrive(void)
{
  char Retx;
  asm {
    mov ax, 25
    int 0x30
    mov Retx, ax
  }
  return Retx;
}

////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////

void GetErrorMessage(int Err, char* msg)
{
  switch (Err) {
    case -0x01:
      msg = "INVALID FUNCTION";
      break;
    case -0x02:
      msg = "FILE NOT FOUND";
      break;
    case -0x03:
      msg = "PATH NOT FOUND";
      break;
    case -0x04:
      msg = "TOO MANY OPEN FILES";
      break;
    case -0x05:
      msg = "ACCESS DENIED";
      break;
    case -0x06:
      msg = "INVALID HANDLE";
      break;
    case -0x0B:
      msg = "FORMAT INVALID";
      break;
    case -0x0C:
      msg = "ACCESS CODE INVALID";
      break;
    case -0x0D:
      msg = "DATA INVALID";
      break;
    case -0x0F:
      msg = "INVALID DRIVE";
      break;
    case -0x10:
      msg = "RMDIR CURRENT";
      break;
    case -0x11:
      msg = "NOT SAME DEVICE";
      break;
    case -0x12:
      msg = "NO MORE FILES";
      break;
    case -0x13:
      msg = "WRITE PROTECTED";
      break;
    case -0x14:
      msg = "UNKNOWN UNIT";
      break;
    case -0x15:
      msg = "DRIVE NOT READY";
      break;
    case -0x17:
      msg = "CRC ERROR";
      break;
    case -0x19:
      msg = "INVALID SEEK";
      break;
    case -0x1A:
      msg = "UNKNOWN MEDIA";
      break;
    case -0x1B:
      msg = "SECTOR NOT FOUND";
      break;
    case -0x1D:
      msg = "WRITE FAULT";
      break;
    case -0x1E:
      msg = "READ FAULT";
      break;
    case -0x1F:
      msg = "GENERAL FAILURE";
      break;
    case -0x20:
      msg = "SHARING VIOLATION";
      break;
    case -0x21:
      msg = "LOCK VIOLATION";
      break;
    case -0x22:
      msg = "INVALID DISK CHANGE";
      break;
    case -0x26:
      msg = "OUT OF INPUT";
      break;
    case -0x27:
      msg = "DISK FULL";
      break;
    case -0x50:
      msg = "FILE EXISTS";
      break;
    case -0x52:
      msg = "CANNOT MAKE DIR";
      break;
    case -0x53:
      msg = "FAIL ON INT24";
      break;
    case -0xB0:
      msg = "NOT LOCKED";
      break;
    case -0xB1:
      msg = "LOCKED IN DRIVE";
      break;
    case -0xB2:
      msg = "NOT REMOVABLE";
      break;
    case -0xB4:
      msg = "LOCK COUNT EXCEEDED";
      break;
    case -0xB5:
      msg = "EJECT FAILED";
      break;
    default:
      msg = "UNKNOWN ERROR #";
      itoa(Err, &msg[14], 10);
      break;
  }
}
