/*  mp3player main software module

    2003-09-27: ad, initial implementation.

    Copyright (c)2003 by Mathias Adam, m.adam@adamis.de

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version. Read the
    full License at http://www.gnu.org/copyleft for more details.
*/

// include files -----
//
#include <mem.h>
#include <malloc.h>
#include <stdio.h>
#include <math.h>
#include <conio.h>
#include <string.h>
#include <io.h>
#include <stdlib.h>
#include <dir.h>

#include "clib/rtos.h"
#include "tk/checkbios.h"
#include "tk/mpbuf.h"
#include "tk/mpeg.h"
#include "tk/vs1001.h"

#include "fat32/fatfs.cpp"
#include "suart.h"
#include "changer.h"


// read last file ID from GNUnilink device
int getLastFileID()
{
  unsigned char ch_userbuf0, ch_userbuf1;
  if (!chg_ReadRegister(8, &ch_userbuf0)) return -1;
  if (!chg_ReadRegister(9, &ch_userbuf1)) return -1;
  return (ch_userbuf1 << 8) | ch_userbuf0;
}

// read last playing position (file offset in ca. ??kbytes [10 Sekunden wre OK]) from GNUnilink device
int getLastFilePos()
{
  unsigned char ch_userbuf2, ch_userbuf3;
  if (!chg_ReadRegister(10, &ch_userbuf2)) return -1;
  if (!chg_ReadRegister(11, &ch_userbuf3)) return -1;
  return (ch_userbuf3 << 8) | ch_userbuf2;
}


struct dbTable {
//  int h;        // fd32 file handle
  FILE *h;
  int Count;    // record count
  int Index;    // current record (0..count-1)
  int RecordSize;
  void *CurrentRecord;
};

int dbSetIndex(dbTable *tbl, int NewIndex);

int dbOpenTable(char *filename, dbTable *tbl, int RecordSize)
{
//  fd32_fs_dosfind_t searchrec;
  ffblk searchrec;

//  if (!fs_dos_findfirst(filename, FD32_FIND_ALLOW_ALL & ~FD32_FIND_ALLOW_VOLUMEID & ~FD32_FIND_ALLOW_DIRECTORY, &searchrec)) {
  if (!findfirst(filename, &searchrec, 0/*FA_RDONLY | FA_HIDDEN | FA_SYSTEM | FA_ARCH*/)) {
//    if ((tbl->h = fs_open(filename, FD32_OPEN_EXIST_OPEN | FD32_OPEN_ACCESS_READ_NA, NULL, 0x27, 1)) < 0)
    if (!(tbl->h = fopen(filename, "rb")))
      return -1;
    tbl->RecordSize = RecordSize;
    tbl->Count = searchrec.ff_fsize /*searchrec.Size*/ / tbl->RecordSize;
    tbl->Index = -1;
    tbl->CurrentRecord = malloc(tbl->RecordSize);
    dbSetIndex(tbl, 0);
  } else return -1;
  return 0;
}

int dbCloseTable(dbTable *tbl)
{
  fclose(tbl->h);
  free(tbl->CurrentRecord);
}

int dbSetIndex(dbTable *tbl, int NewIndex)
{
  if ((NewIndex < 0) || (NewIndex >= tbl->Count)) return -1;
  if (NewIndex == tbl->Index) return 0;
  tbl->Index = NewIndex;
  long Offset = tbl->Index * tbl->RecordSize;
//  fs_lseek(tbl->h, &Offset, 0);
  if (fseek(tbl->h, Offset, SEEK_SET)) return -1;
  unsigned long BlockSize = tbl->RecordSize;
//  fs_read(tbl->h, tbl->CurrentRecord, &BlockSize);
  fread(tbl->CurrentRecord, BlockSize, 1, tbl->h);
  return 0;
}


struct FilesRec {       // all strings are null-terminated!
  char Artist[65];
  char Album[65];
  char Title[64];
  unsigned __int8 TrackNum;
  char Comment[65];
  char Year[5];
};

int main(int argc, char* argv[])
{
  long m_old, s_old;

  if (!CheckBIOS())
    return -1;

// "global" variables
  dbTable tblFiles;
  FilesRec *recFile;

// initialization
  // mp3 file database
  if (dbOpenTable("A:\\FILES.DB", &tblFiles, sizeof(FilesRec))) {
    printf("Cannot open table B:\\FILES.DB!\n");
    return -1;
  }
  recFile = (FilesRec*)tblFiles.CurrentRecord;
  printf("table FILES: Count=%i, Index=%i, RecordSize=%i\n", tblFiles.Count, tblFiles.Index, tblFiles.RecordSize);
  for (int i = 0; i < tblFiles.Count; i++) {
    dbSetIndex(&tblFiles, i);
    printf("  %.3i %s - %s - %s\n", i, recFile->Artist, recFile->Album, recFile->Title);
  }

  // VS1001 mp3 decoder chip
  VS_Init(15000000);     // clock speed 15MHz

  // autochanger (unilink) interface
  suart_Init(6, 5);      // set RX = PIO??, TX = PIO??

  // others
  BUF_Init(16 * 1024, 128 * 1024L);
  randomize();


// restore play position, set play mode, ...
  int lastID = getLastFileID();
  printf("getLastFileID()=%i\n", lastID);
  dbSetIndex(&tblFiles, lastID);
  int lastPos = -1;//  /*48; //-1; //*/getLastFilePos();
  printf("getLastFilePos()=%i\n", lastPos);
  unsigned int lastPosWritten = lastPos;   // speichert zuletzt geschriebenen Wert, damit nicht zu hufig
      // neu in PIC geschrieben werden muss  - evtl. diese Funktionalitt in chg_WriteReg einbauen??  - 031001


// main loop
  enum _Playmode { PLAY, REPEAT, RANDOM, REPEATGROUP, RANDOMGROUP /*,INTRO*/ ,LASTPLAYMODE };
  unsigned char last_ch_rxdisk = 0, last_ch_rxtrack = 0, last_ch_rxplaymode = 0xFF;
  int Playmode = PLAY;
  bool Unilink = true;    // can be turned off via telnet ('u') to avoid timeouts when not in car

  if (!chg_WriteRegister(0, (tblFiles.Count + 98) / 99))     // DiskCount
;//    printf("write DiskCount failed!\n");
  while (tblFiles.Index >= 0) {    // index is set to -1 on key stroke (loop normally never ends in car use)
  // update display
    if (!chg_WriteRegister(1, (tblFiles.Index / 99) + 1))      // DiskNum
;//      printf("write DiskNum failed!\n");
    if ((tblFiles.Index / 99) < ((tblFiles.Count - 1) / 99)) {
      if (!chg_WriteRegister(4, 99))                      // TrackCount
;//        printf("write TrackCount failed!\n");
    } else {
      if (!chg_WriteRegister(4, ((tblFiles.Count - 1) % 99) + 1)) // TrackCount
;//        printf("write TrackCount failed!\n");
    }
    if (!chg_WriteRegister(5, (tblFiles.Index % 99) + 1))  // TrackNum
;//      printf("write TrackNum failed!\n");
    if (!chg_WriteRegister(0x3f, 1))                      // update display: DiskNum
;//      printf("write Display[1] failed!\n");

  // play file
    char mp3filename[260];
    strcpy(mp3filename, "C:\\");
    strcat(mp3filename, recFile->Artist);
    strcat(mp3filename, "\\");
    if (recFile->Album[0]) {
      strcat(mp3filename, recFile->Album);
      strcat(mp3filename, "\\");
    }
    if (recFile->TrackNum) {
      char chtracknum[5];
      sprintf(chtracknum, "%.2hu ", recFile->TrackNum);
      strcat(mp3filename, chtracknum);
    }
    strcat(mp3filename, recFile->Title);
    strcat(mp3filename, ".mp3");
    printf("Starting playback: %s\n", mp3filename);

    // show track name (up to 8 chars) on car radio display:
    if (Unilink) chg_SendDiskName(recFile->Title);

    int mp3file;
    if ((mp3file = fs_open(mp3filename, FD32_OPEN_EXIST_OPEN | FD32_OPEN_ACCESS_READ, NULL, 0x27, 1)) >= 0) {
      VS_Reset();
      BUF_Play();
      bool IsPlaying = true;
      long fpos = 0;
      // seek to last playing position (only for very first file played):
      if (lastPos >= 0) {
        fpos = lastPos * 65536;
        printf("seeking to: %li\n", fpos);
        if (fs_lseek(mp3file, &fpos, FD32_SEEK_SET))
          puts("fs_lseek to lastPos failed!");
        printf("fpos=%li\n", fpos);
        lastPos = -1;
//        lastPos++;   mal zum Testen benutzt - wg. gelegentlichem Absturz bei fs_lseek  (031002)
      }
      printf("bitrate  file_pos   time  time[from VS1001]\n");
		while (IsPlaying) {
      // Read data from mp3 file
        unsigned long len;
        long len1, len2;
        char *buf1, *buf2;

        BUF_Lock(BUF_FROMWRITEPOS, min(16384L, BUF_GetFreeBytes()), &buf1, &len1, &buf2, &len2);
        len = len1;
        if (fs_read(mp3file, buf1, &len) < 0)   // re-sets len to actual number of bytes read
          puts("fs_read 1 failed!");
        if (buf2 && (len == len1)) {    // 2nd buffer available and eof not yet reached
          len = len2;
          if (fs_read(mp3file, buf2, &len) < 0)   // re-sets len to actual number of bytes read
            puts("fs_read 2 failed!");
          len += len1;
        }
        fpos += len;
        BUF_Unlock(len);

      // display time on telnet and headunit
        unsigned long header;
        VS_GetMPEGHeader(&header);
        FrameInfo info;
        if (MPEG_CheckHeader(header) && MPEG_ParseHeader(header, &info))
        {
          long s, m;
          unsigned int VS_s, VS_m;
          s = ((fpos - BUF_GetCount()) / 128) / (info.bitrate/1000);
          m = s / 60;       //calc minute
          s = s % 60;       //calc restseconds
          VS_s = VS_GetDecodeTime();
          VS_m = VS_s / 60;
          VS_s = VS_s % 60;
          printf("\r   %.3lik  %.8li  %.2li:%.2li  %.2u:%.2u", info.bitrate/1000, fpos-BUF_GetCount(),
            m, s,  VS_m, VS_s);

          // show time on car radio display:
          if (Unilink) {
            if (VS_m > 0xFF) VS_m = 0xFF;
            if (VS_m < 0) VS_m = 0;
            if (VS_s < 0) VS_s = 0;
//            ch_send_time(m, s);
            if ((VS_m != m_old) || (VS_s != s_old)) {
              m_old = VS_m;
              s_old = VS_s;

              chg_WriteRegister(7, VS_m);
              chg_WriteRegister(6, ((VS_s / 10) << 4) | (VS_s % 10));

/*              chg_WriteRegister(0x3f, 3);        // update Counter (and disk name if "4" was sent before
                     // dies besser vor SendDiskName, da sonst Zeitanzeige etwas unregelmig
                     // (weil SendDiskName recht lange braucht). So wird nun bei Sekunde 4,7,10,...
                     // gescrollt                           - ad/0310031745     */

              // doch besser so machen:   (0310031850)
              if (VS_s % 3 == 0) {     // every 3 seconds
                chg_WriteRegister(0x3f, 4);        // update disk name
                chg_WriteRegister(0x3f, 3);        // update Counter and start transmission to head unit
              } else
                chg_WriteRegister(0x3f, 3);        // update Counter only

              if (VS_s % 3 == 2) {     // send new disk name to GNUnilink 1 second before
                int namelen = strlen(recFile->Title);
/*                if ((namelen > 8) && (namelen <= 16)) {
                  int nameindex = ((VS_s / 3) % 2) == 0 ? 0 : namelen-8;
                  printf("nameindex=%i", nameindex);
                  chg_SendDiskName(&recFile->Title[nameindex]);
                  chg_WriteRegister(0x3f, 1);        // update Counter
                }  */
                const DisplayLength = 8;    // number of characters the display can show simultaneously
                const ScrollPeriod = 3;     // scroll every ... seconds
                const ScrollAmount = 8;     // number of characters to advance each time (<= DisplayLength)
//                if (namelen > DisplayLength) {                auch kurze Namen regelmig neu senden - ad/031003
                  int nameindex = (((VS_s+1) / ScrollPeriod) % ((namelen+ScrollAmount-1) / ScrollAmount)) * ScrollAmount;
                  chg_SendDiskName(&recFile->Title[nameindex]);
//nein,s.o. (0310031850)                  chg_WriteRegister(0x3f, 4);        // update Counter
//                }
              }
            }
          }
        }

        if (Unilink) {
        // save current playing position regularly   (not yet used as fs_lseek seems to crash very often)
          if (((fpos-BUF_GetCount()) >> 16) != lastPosWritten) {
            lastPosWritten = fpos >> 16;
            chg_WriteRegister(10, (lastPosWritten & 0xFF));       // write register 10 (UserBuf2)
            chg_WriteRegister(11, lastPosWritten >> 8);           // write register 11 (UserBuf3)
          }

        // poll GNUnilink for control commands
          unsigned char ch_rxdisk, ch_rxtrack, ch_rxtrackRel, ch_rxplaymode, ch;
          if (chg_ReadRegister(12, &ch_rxdisk) && ch_rxdisk) {    // DiskNum received
            chg_WriteRegister(12, 0);                // clear register
            if (!dbSetIndex(&tblFiles, (ch_rxdisk - 1) * 99))
              break;                                 // stop current song
          }
          if (chg_ReadRegister(13, &ch_rxtrack) && ch_rxtrack) {   // TrackNum received
            chg_WriteRegister(13, 0);                // clear register
            if (!dbSetIndex(&tblFiles, (tblFiles.Index / 99) * 99 + ch_rxtrack - 1))
              break;                                 // stop current song
          }
          if (chg_ReadRegister(15, &ch_rxplaymode)) {  // PlayMode received
/*            if (ch_rxplaymode & 0x01) {                // RPT mode selected
              PlayMode = REPEATGROUP;		  // Wiederhole Verzeichnis
            } else*/ if (ch_rxplaymode & 0x02) {         // RP mode selected
              Playmode = RANDOM;    	      	  // Zufall (ber alle Dateien)
            } else {                						  // normal mode
              Playmode = PLAY;              // alle Titel nacheinander
            }
          }
          if (chg_ReadRegister(14, &ch_rxtrackRel) && ch_rxtrackRel) {   // TrackRel received
            chg_WriteRegister(14, 0);     // clear register 14 (TrackRel)
            Playmode = RANDOM;                        // Becker benutzt dies nur im RP-Modus
                     // Achtung: Playmode wird bei nchster nderung am Playmode-Register im
                     // PIC wieder zurckgesetzt! Besser: PIC sollte sein Playmode-Bit
                     // entsprechend setzen, wenn er einen TrackRel-Befehl empfngt; oder
                     // noch besser: PIC stabilisieren, damit er alles vom Radio korrekt
                     // empfngt                                             - 031001
            dbSetIndex(&tblFiles, random(tblFiles.Count));
            printf("\nnext track was selected by TrackRel\n");
            break;
          }
        }

        // be nice to other apps
        RTX_Sleep_Time(100);

        if (kbhit()) {
          switch (getch()) {
            case 'u':
              Unilink = !Unilink;
              printf("\nUnilink communication now %sabled.\n", Unilink ? "en" : "dis");
              break;

            case 'r':
              Playmode = ((Playmode == PLAY) ? RANDOM : PLAY);
              printf("\nNew playmode: %s\n", (Playmode == PLAY) ? "PLAY" : "RANDOM");
              break;

            case 't':
              printf("\nEnter new track number (0..%i): ", tblFiles.Count-1);
              int iNewTrack;
              scanf("%i", &iNewTrack);
              dbSetIndex(&tblFiles, iNewTrack);
              IsPlaying = false;
              break;

            default:
              tblFiles.Index = -1;   // stop playback
              IsPlaying = false;
          }
//          break;    // beende while Schleife (knnte auch durch Setzen von IsPlaying und IsPlaying = IsPlaying && ...
             // unten gelst werden, mal sehen was besser ist  /  Isplaying etvtl. in !SongFinished umbenennen) - 030930
        }
        if (!IsPlaying) break;
        IsPlaying = IsPlaying && ((len == len1+len2) || (BUF_GetState() != BUF_STALLED));  // das IsPlaying && ist eigentlich doch unntig
        if (!IsPlaying) {   // song finished normally - determine next song
          switch (Playmode) {
            case PLAY:
              if (dbSetIndex(&tblFiles, tblFiles.Index+1))
                dbSetIndex(&tblFiles, 0);                      // select track 0 after last track finished
              break;

            case RANDOM:
              dbSetIndex(&tblFiles, random(tblFiles.Count));
              break;
          }
        }
      }
      BUF_Clear();
      fs_close(mp3file);
      puts("");  // newline
    }
    //playing of file finished; tblfiles.index has been set to new file index during play file

    if (tblFiles.Index >= 0) {      // save track ID (only if != -1)
      if (!chg_WriteRegister(8, (tblFiles.Index & 0xFF)))       // write register 8 (UserBuf0)
;//      printf("write UserBuf0 failed!\n");
      if (!chg_WriteRegister(9, tblFiles.Index >> 8))  // write register 9 (UserBuf1)
;//      printf("write UserBuf1 failed!\n");
    }
  }


// clean up
  dbCloseTable(&tblFiles);
  BUF_Done();
  VS_Done();

  puts("MP3.EXE finished.\n");
}
