/*  Software-UART (RS232-compatible)

  ad/031002: hint: only suart_Init, suart_Tx90RxChar and suart_Tx91RxChar are
             currently being used by mp3.exe
  ad/021125: initial implementation.
             Fixed baudrate of 78125bd.

    Copyright (c)2002 by Mathias Adam, m.adam@adamis.de

	  This program is free software; you can redistribute it and/or
	  modify it under the terms of the GNU General Public License as
	  published by the Free Software Foundation; either version 2 of
	  the License, or (at your option) any later version. Read the
	  full License at http://www.gnu.org/copyleft for more details.

*/

// include files -----
//
#include <conio.h>
#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <stdlib.h>
#include <stdarg.h>
#include <assert.h>
#include "amd186.h"
#include "suart.h"


static unsigned long RxDPio_Mask, TxDPio_Mask;
static unsigned int RxDPio_PDReg, TxDPio_PDReg;
static unsigned char RxDPio_MaskByte, TxDPio_MaskByte;


/**
* assign PIOs that are to be used as RxD/TxD-Pins, init the selected PIOs
* and prepare mask values to speed up the actual transfer routines
* Values for RxDPIO/TxDPIO are the Beck PIO numbers (0..13); you cannot
* use the same PIO for both RxDPIO and TxDPIO!
*/
void suart_Init(unsigned char RxDPio, unsigned char TxDPio)
{
  RxDPio_Mask = AMD_PIOMASK[RxDPio];
  TxDPio_Mask = AMD_PIOMASK[TxDPio];
  if (RxDPio_Mask & 0xFF) {
    RxDPio_PDReg = PCB_PDATA0;
    RxDPio_MaskByte = RxDPio_Mask & 0xFF;
  } else if (RxDPio_Mask & 0xFF00) {
    RxDPio_PDReg = PCB_PDATA0+1;
    RxDPio_MaskByte = (RxDPio_Mask >> 8L) & 0xFF;
  } else if (RxDPio_Mask & 0xFF0000) {
    RxDPio_PDReg = PCB_PDATA1;
    RxDPio_MaskByte = (RxDPio_Mask >> 16L) & 0xFF;
  } else {
    RxDPio_PDReg = PCB_PDATA1+1;
    RxDPio_MaskByte = (RxDPio_Mask >> 24L) & 0xFF;
  }
  if (TxDPio_Mask & 0xFF) {
    TxDPio_PDReg = PCB_PDATA0;
    TxDPio_MaskByte = TxDPio_Mask & 0xFF;
  } else if (TxDPio_Mask & 0xFF00) {
    TxDPio_PDReg = PCB_PDATA0+1;
    TxDPio_MaskByte = (TxDPio_Mask >> 8) & 0xFF;
  } else if (TxDPio_Mask & 0xFF0000) {
    TxDPio_PDReg = PCB_PDATA1;
    TxDPio_MaskByte = (TxDPio_Mask >> 16) & 0xFF;
  } else {
    TxDPio_PDReg = PCB_PDATA1+1;
    TxDPio_MaskByte = (TxDPio_Mask >> 24) & 0xFF;
  }

  AMD_SetPioMode(RxDPio_Mask | TxDPio_Mask, 0xFFFFFFFF);    // Mode=1: output/input wo pullup/down
  AMD_SetPioDir(RxDPio_Mask, 0xFFFFFFFF);                    // input
  AMD_SetPioDir(TxDPio_Mask, 0);                             // output
  AMD_SetPioData(TxDPio_Mask, 0xFFFFFFFF);                   // TxD must be high when idle
}


/**
* Send out one byte on TxD line
* Interrupts are disabled while this routine runs as timing is rather critical
*/
unsigned int a[33];
void suart_TxChar(unsigned char tx_char)
{  unsigned int ddd;         //return ;
//printf("(%d)  ", tx_char);

  asm {
//   db   0xAD,0xAD,0xAD,0xAD,0xAD,0xAD,0xAD,0xAD;
    cli																						// fa

    mov   dx, word ptr TxDPio_PDReg													// 88-8c,8e,a0-a3,b0+,b8+,c6,c7
    mov   bh, byte ptr TxDPio_MaskByte
    mov   bl, bh
    not   bl                           											// f6
    in    al, dx                                                        // ec
    mov   ah, byte ptr tx_char

//mov   si, 0xAD
//push  si
    mov   cx, 10                // 8 bits + start/stop bits
//mov ddd, cx
    clc                         // start bit polarity = 0               // f8
out_bit:
    jc    bit_1                 // 13,4 clocks                          // 72
//mov   si, 0
//push  si
    and   al, bl                // 3 clocks;   clears carry!            // 20/22
    rcr   ah, 1                 // waste time; 2x2+3 clocks             // d0
    rcl   ah, 1                                                         // d0
    nop                                                                 // 90
    jmp   do_bit                // 14 clocks	  									// eb
bit_1:
//mov   si, 1
//push  si
    or    al, bh                // 3 clocks;   clears carry!            // 08
    nop                         // waste time; 4x3 clocks
    nop
    nop
    nop
do_bit:
    out   dx, al                // 7 clocks                             // ee
                        // since out_bit: 35 clocks
    mov   di, cx                // 9/12(?) clocks                       // 89
    mov   cx, 12                // 3 clocks                             // b9
delay_loop:
    loop  delay_loop            // (cx-1)*16+6 clocks                   // e2
    mov   cx, di                // 9/12(?) clocks                       // 89
                        // delay loop: 27+(cx-1)*16 clocks
    stc                         // stop bit polarity = 1 (rotate '1's into ah so 10th read bit is '1')      // f9
                                // stc: 2 clocks - those are too much, but timing is not THAT critical...
    rcr   ah, 1                 // 2 clocks                             // d0
    loop  out_bit               // 16,6 clocks

/*pop  si
mov  word ptr a[0], si
pop  si
mov  word ptr a[2], si
pop  si
mov  word ptr a[4], si
pop  si
mov  word ptr a[6], si
pop  si
mov  word ptr a[8], si
pop  si
mov  word ptr a[10], si
pop  si
mov  word ptr a[12], si
pop  si
mov  word ptr a[14], si
pop  si
mov  word ptr a[16], si
pop  si
mov  word ptr a[18], si
pop  si
mov  word ptr a[20], si  */
    or    al, bh                // go idle (stop bit is a little shorter than other bits)
    out   dx, al

    sti                                                                 // fb
  }
//printf("(%hx)\n", ddd);
//printf("%hx, %hx, %hx, %hx, %hx, %hx, %hx, %hx, %hx, %hx, %hx\n", a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8],a[9],a[10]);
}

/**
* Send out one byte on TxD line
* Interrupts are disabled while this routine runs as timing is rather critical
*/
void suart_TxChar_90(unsigned char tx_char)
{
  asm {
    cli

    mov   dx, word ptr TxDPio_PDReg
    mov   bh, byte ptr TxDPio_MaskByte
    mov   bl, bh
    not   bl
    in    al, dx
    mov   ah, byte ptr tx_char

    mov   cx, 9                 // 8 bits + start bit
    clc                         // start bit polarity = 0
out_bit:
    jc    bit_1                 // 13,4 clocks
    and   al, bl                // 3 clocks;   clears carry!
    rcr   ah, 1                 // waste time; 2x2+3 clocks
    rcl   ah, 1
    nop
    jmp   do_bit                // 14(?) clocks
bit_1:
    or    al, bh                // 3 clocks;   clears carry!
    nop                         // waste time; 4x3 clocks
    nop
    nop
    nop
do_bit:
    out   dx, al                // 7 clocks 
                        // from out_bit: 35 clocks
    mov   di, cx                // 9/12(?) clocks
    mov   cx, 12                // 3 clocks
delay_loop:
    loop  delay_loop            // (cx-1)*16+6 clocks 
    mov   cx, di                // 9/12(?) clocks
                        // delay loop: 27+(cx-1)*16 clocks 
    stc                         // stop bit polarity = 1 (rotate '1's into ah so 10th read bit is '1')
                                // 2 clocks - those are too much, but timing is not THAT critical...
    rcr   ah, 1                 // 2 clocks
    loop  out_bit               // 16,6 clocks

// send out ninth data bit (=0)
    and   al, bl                // 3 clocks;   clears carry!
    out   dx, al                // 7 clocks
    mov   cx, 16                // 3 clocks
delay_loop_2:
    loop  delay_loop_2          // (cx-1)*16+6 clocks

// send stop bit (=1)
    or    al, bh                // 3 clocks;   clears carry!
    out   dx, al                // 7 clocks
    mov   cx, 16                // 3 clocks
delay_loop_3:
    loop  delay_loop_3          // (cx-1)*16+6 clocks

/*wozu das?
    or    al, bh                // go idle (stop bit is a little shorter than other bits)
    out   dx, al*/

    sti
  }
}

/**
* Send out one byte on TxD line
* Interrupts are disabled while this routine runs as timing is rather critical
*/
void suart_TxChar_91(unsigned char tx_char)
{
  asm {
    cli

    mov   dx, word ptr TxDPio_PDReg
    mov   bh, byte ptr TxDPio_MaskByte
    mov   bl, bh
    not   bl
    in    al, dx
    mov   ah, byte ptr tx_char

    mov   cx, 9                 // 8 bits + start bit
    clc                         // start bit polarity = 0
out_bit:
    jc    bit_1                 // 13,4 clocks
    and   al, bl                // 3 clocks;   clears carry!
    rcr   ah, 1                 // waste time; 2x2+3 clocks
    rcl   ah, 1
    nop
    jmp   do_bit                // 14(?) clocks
bit_1:
    or    al, bh                // 3 clocks;   clears carry!
    nop                         // waste time; 4x3 clocks
    nop
    nop
    nop
do_bit:
    out   dx, al                // 7 clocks
                        // from out_bit: 35 clocks
    mov   di, cx                // 9/12(?) clocks
    mov   cx, 12                // 3 clocks
delay_loop:
    loop  delay_loop            // (cx-1)*16+6 clocks
    mov   cx, di                // 9/12(?) clocks
                        // delay loop: 27+(cx-1)*16 clocks
    stc                         // stop bit polarity = 1 (rotate '1's into ah so 10th read bit is '1')
                                // 2 clocks - those are too much, but timing is not THAT critical...
    rcr   ah, 1                 // 2 clocks
    loop  out_bit               // 16,6 clocks

// send out ninth data bit (=1)
    or    al, bh                // 3 clocks;   clears carry!
    out   dx, al                // 7 clocks
    mov   cx, 16                // 3 clocks
delay_loop_2:
    loop  delay_loop_2          // (cx-1)*16+6 clocks

// send stop bit (=1)
    or    al, bh                // 3 clocks;   clears carry!
    out   dx, al                // 7 clocks
    mov   cx, 16                // 3 clocks
delay_loop_3:
    loop  delay_loop_3          // (cx-1)*16+6 clocks

/*wozu das?
    or    al, bh                // go idle (stop bit is a little shorter than other bits)
    out   dx, al*/

    sti
  }
}


/**
* Wait up to 100ms for a character to receive; return received character and true if
* encountered one within the timeout; return false otherwise.
* Interrupts are disabled while this routine runs as timing is rather critical
*/
bool suart_RxChar(unsigned char *rx_char)
{
  bool resu = true;
  unsigned char test_var;
  asm {
    cli

    mov   dx, word ptr RxDPio_PDReg
    mov   bh, byte ptr RxDPio_MaskByte

    mov   cx, 0
wait_for_start:         // timeout = ca. 31*cx clocks
    in    al, dx                // 8 clocks
    and   al, bh                // 3 clocks;   clears carry!
    jz    start                 // 13,4 clocks
    loop  wait_for_start        // 16,6 clocks

    mov   byte ptr resu, 0
    jmp   finished

start:
    mov   cx, 2                 // 3 clocks
short_delay:
    loop  short_delay           // 16,6 clocks

    mov   cx, 8                 // load 8 bits (simply ignore stop bit)
in_bit:
// wait one bit length-44clks (1/baudrate sec.)-44:
// 27+(cx-1)*16+44=20E6/baudrate --> cx:=(((20E6/baudrate)-71)/16)+1=12,56
    mov   di, cx                // 9/12(?) clocks
    mov   cx, 13                // 3 clocks
bit_delay:
    loop  bit_delay             // 16,6 clocks
    nop; nop; nop/*; nop; nop*/
    mov   cx, di                // 9/12(?) clocks

    in    al, dx                // 8 clocks
    and   al, bh                // 3 clocks
    jnz   bit_1                 // 13,4 clocks
    clc                         // 2 clocks
    rcr   ah, 1                 // 2 clocks
    nop; nop; nop               // waste 9 clocks
    loop  in_bit                // 16,6 clocks
    jmp   finished              // 14 clocks
bit_1:
    stc                         // 2 clocks
    rcr   ah, 1                 // 2 clocks
    loop  in_bit                // 16,6 clocks
finished:
/*maybe: check stop bit; return false if stop_bit=0*/
//    mov   *(byte ptr rx_char), ah
    mov   byte ptr test_var, ah

    sti
  }
//  printf("test: %i\n", test_var);
  *rx_char = test_var;
  return resu;
}



bool suart_Tx90RxChar(unsigned char tx_char, unsigned char *rx_char)
{
  bool resu = true;
  unsigned char test_var;
  asm {
// code from TxChar_91
    cli

    mov   dx, word ptr TxDPio_PDReg
    mov   bh, byte ptr TxDPio_MaskByte
    mov   bl, bh
    not   bl
    in    al, dx
    mov   ah, byte ptr tx_char

    mov   cx, 9                 // 8 bits + start bit
    clc                         // start bit polarity = 0
out_bit:
    jc    tx_bit_1              // 13,4 clocks
    and   al, bl                // 3 clocks;   clears carry!
    rcr   ah, 1                 // waste time; 2x2+3 clocks
    rcl   ah, 1
    nop
    jmp   do_bit                // 14(?) clocks
tx_bit_1:
    or    al, bh                // 3 clocks;   clears carry!
    nop                         // waste time; 4x3 clocks
    nop
    nop
    nop
do_bit:
    out   dx, al                // 7 clocks
                        // from out_bit: 35 clocks
    mov   di, cx                // 9/12(?) clocks
    mov   cx, 12                // 3 clocks
delay_loop:
    loop  delay_loop            // (cx-1)*16+6 clocks
    mov   cx, di                // 9/12(?) clocks
                        // delay loop: 27+(cx-1)*16 clocks
    stc                         // stop bit polarity = 1 (rotate '1's into ah so 10th read bit is '1')
                                // 2 clocks - those are too much, but timing is not THAT critical...
    rcr   ah, 1                 // 2 clocks
    loop  out_bit               // 16,6 clocks

// send out ninth data bit (=0)
    and   al, bl                // 3 clocks;   clears carry!
    out   dx, al                // 7 clocks
    mov   cx, 16                // 3 clocks
delay_loop_2:
    loop  delay_loop_2          // (cx-1)*16+6 clocks

// send stop bit (=1)
    or    al, bh                // 3 clocks;   clears carry!
    out   dx, al                // 7 clocks
    // no delay necessary here due to the following receive loop

// begin RxChar code
    mov   dx, word ptr RxDPio_PDReg
    mov   bh, byte ptr RxDPio_MaskByte

    mov   cx, 600
wait_for_start:         // timeout = ca. 31*cx clocks
    in    al, dx                // 8 clocks
    and   al, bh                // 3 clocks;   clears carry!
    jz    start                 // 13,4 clocks
    loop  wait_for_start        // 16,6 clocks

    mov   byte ptr resu, 0
    jmp   finished

start:
    mov   cx, 2                 // 3 clocks
short_delay:
    loop  short_delay           // 16,6 clocks

    mov   cx, 8                 // load 8 bits (simply ignore stop bit)
in_bit:
// wait one bit length-44clks (1/baudrate sec.)-44:
// 27+(cx-1)*16+44=20E6/baudrate --> cx:=(((20E6/baudrate)-71)/16)+1=12,56
    mov   di, cx                // 9/12(?) clocks
    mov   cx, 13                // 3 clocks
bit_delay:
    loop  bit_delay             // 16,6 clocks
    nop; nop; nop/*; nop; nop*/
    mov   cx, di                // 9/12(?) clocks

    in    al, dx                // 8 clocks
    and   al, bh                // 3 clocks
    jnz   rx_bit_1              // 13,4 clocks
    clc                         // 2 clocks
    rcr   ah, 1                 // 2 clocks
    nop; nop; nop               // waste 9 clocks
    loop  in_bit                // 16,6 clocks
    jmp   finished              // 14 clocks
rx_bit_1:
    stc                         // 2 clocks
    rcr   ah, 1                 // 2 clocks
    loop  in_bit                // 16,6 clocks
finished:
/*maybe: check stop bit; return false if stop_bit=0*/
//    mov   *(byte ptr rx_char), ah
    mov   byte ptr test_var, ah

    sti
  }
//  printf("test: %i\n", test_var);
  *rx_char = test_var;
  return resu;
}

bool suart_Tx91RxChar(unsigned char tx_char, unsigned char *rx_char)
{
  bool resu = true;
  unsigned char test_var;
  asm {
// code from TxChar_91
    cli

    mov   dx, word ptr TxDPio_PDReg
    mov   bh, byte ptr TxDPio_MaskByte
    mov   bl, bh
    not   bl
    in    al, dx
    mov   ah, byte ptr tx_char

    mov   cx, 9                 // 8 bits + start bit
    clc                         // start bit polarity = 0
out_bit:
    jc    tx_bit_1              // 13,4 clocks
    and   al, bl                // 3 clocks;   clears carry!
    rcr   ah, 1                 // waste time; 2x2+3 clocks
    rcl   ah, 1
    nop
    jmp   do_bit                // 14(?) clocks
tx_bit_1:
    or    al, bh                // 3 clocks;   clears carry!
    nop                         // waste time; 4x3 clocks
    nop
    nop
    nop
do_bit:
    out   dx, al                // 7 clocks
                        // from out_bit: 35 clocks
    mov   di, cx                // 9/12(?) clocks
    mov   cx, 12                // 3 clocks
delay_loop:
    loop  delay_loop            // (cx-1)*16+6 clocks
    mov   cx, di                // 9/12(?) clocks
                        // delay loop: 27+(cx-1)*16 clocks
    stc                         // stop bit polarity = 1 (rotate '1's into ah so 10th read bit is '1')
                                // 2 clocks - those are too much, but timing is not THAT critical...
    rcr   ah, 1                 // 2 clocks
    loop  out_bit               // 16,6 clocks

// send out ninth data bit (=1)
    or    al, bh                // 3 clocks;   clears carry!
    out   dx, al                // 7 clocks
    mov   cx, 16                // 3 clocks
delay_loop_2:
    loop  delay_loop_2          // (cx-1)*16+6 clocks

// send stop bit (=1)
    or    al, bh                // 3 clocks;   clears carry!
    out   dx, al                // 7 clocks
    // no delay necessary here due to the following receive loop

// begin RxChar code
    mov   dx, word ptr RxDPio_PDReg
    mov   bh, byte ptr RxDPio_MaskByte

    mov   cx, 600
wait_for_start:         // timeout = ca. 31*cx clocks
    in    al, dx                // 8 clocks
    and   al, bh                // 3 clocks;   clears carry!
    jz    start                 // 13,4 clocks
    loop  wait_for_start        // 16,6 clocks

    mov   byte ptr resu, 0
    jmp   finished

start:
    mov   cx, 2                 // 3 clocks
short_delay:
    loop  short_delay           // 16,6 clocks

    mov   cx, 8                 // load 8 bits (simply ignore stop bit)
in_bit:
// wait one bit length-44clks (1/baudrate sec.)-44:
// 27+(cx-1)*16+44=20E6/baudrate --> cx:=(((20E6/baudrate)-71)/16)+1=12,56
    mov   di, cx                // 9/12(?) clocks
    mov   cx, 13                // 3 clocks
bit_delay:
    loop  bit_delay             // 16,6 clocks
    nop; nop; nop/*; nop; nop*/
    mov   cx, di                // 9/12(?) clocks

    in    al, dx                // 8 clocks
    and   al, bh                // 3 clocks
    jnz   rx_bit_1              // 13,4 clocks
    clc                         // 2 clocks
    rcr   ah, 1                 // 2 clocks
    nop; nop; nop               // waste 9 clocks
    loop  in_bit                // 16,6 clocks
    jmp   finished              // 14 clocks
rx_bit_1:
    stc                         // 2 clocks
    rcr   ah, 1                 // 2 clocks
    loop  in_bit                // 16,6 clocks
finished:
/*maybe: check stop bit; return false if stop_bit=0*/
//    mov   *(byte ptr rx_char), ah
    mov   byte ptr test_var, ah

    sti
  }
//  printf("test: %i\n", test_var);
  *rx_char = test_var;
  return resu;
}

