﻿
#include <stdio.h>
#include <string.h>
#include <dir.h>
#include <io.h>
#include <fcntl.h>
#include <errno.h>
#include "unicode.h"

#include "fatfs.cpp"

// [from checkbios.cpp]
static void getFocus()
{
  REGPACK regs;
  regs.r_ax = 0x1100 | 2;
  intr( 0xa0, &regs );
}


static void releaseFocus()
{
  REGPACK regs;
  regs.r_ax = 0x1100 | 3;
  intr( 0xa0, &regs );
}





void dirs(char* name)
{
  fd32_fs_dosfind_t searchrec;
  int      done;

  unsigned Datum, Zeit;
  done = fs_dos_findfirst(name, 0x3f, &searchrec);
  while (!done) {
    Datum = searchrec.MDate;
    Zeit = searchrec.MTime;
    printf("%02u.%02u.%02u  ", Datum & 0x1f, (Datum >> 5) & 0x0f, ((Datum >> 9) & 0x7f)+1980);
    printf("%02u:%02u    ", (Zeit >> 11) & 0x1f, (Zeit >> 5) & 0x0f);
    if (searchrec.Attr & FD32_ATTR_DIRECTORY)
      printf("  <DIR>        ");
    else
      printf("%13lu  ", searchrec.Size);
    printf("%s\n", searchrec.Name);

    done = fs_dos_findnext(&searchrec);
  }
}

void dirl(char* name)
{
  fd32_fs_lfnfind_t searchrec;
  DWORD LongName_Scalar[300];
  WORD LongName_UTF16[300];
  char LongName_oem[300];
  int      done;

  unsigned Datum, Zeit;
  done = fs_lfn_findfirst(name, 0x3f, &searchrec);
  while (!done) {
    Datum = searchrec.MTime >> 16;
    Zeit = searchrec.MTime & 0xffff;
    printf("%02u.%02u.%02u  ", Datum & 0x1f, (Datum >> 5) & 0x0f, ((Datum >> 9) & 0x7f)+1980);
    printf("%02u:%02u    ", (Zeit >> 11) & 0x1f, (Zeit >> 5) & 0x0f);
    if (searchrec.Attr & FD32_ATTR_DIRECTORY)
      printf("  <DIR>        ");
    else
      printf("%13lu  ", searchrec.SizeLo);
    // convert LongName from UTF-8 to OEM
/*    if (utf8_to_utf16((char*)&searchrec.LongName, (unsigned int*)&LongName_UTF16))
      printf("UTF16_FAILED!!");
    convert_unicode_string_to_oem(LongName_oem, (unsigned short*)LongName_UTF16);
*/    printf("[%.3i]%s\n", strlen(searchrec.LongName), /*LongName_oem*/searchrec.LongName);

    done = fs_lfn_findnext(&searchrec);
  }
  fs_lfn_findclose(&searchrec);
}

void typeit(char* name)
{
  int   handle;
  char  buffer[8192];
  unsigned long  cnt;
  if( (handle = fs_open(name, FD32_OPEN_EXIST_OPEN | FD32_OPEN_ACCESS_READ, NULL, 0x27, 1)) < 0 )
    printf("Cannot open file. Error %d\n", handle);
  else {
    do {
      cnt = 8191;
      if (fs_read(handle, buffer, &cnt) >= 0) {
        buffer[cnt] = 0;
        printf("%s", &buffer);
      } else
        printf("Error reading file.\n");
    } while (cnt > 0);
  }
  fs_close(handle);
}

void write(char* name)
{
  int handle, res;

  printf("WRITE\n");
  handle = fs_open(name, FD32_OPEN_ACCESS_WRITE | FD32_OPEN_EXIST_OPEN | FD32_OPEN_NOTEXIST_CREATE, NULL, FD32_ATTR_ARCHIVE, 1);
  if (handle < 0) {
    printf("ERROR: %d\n", handle);
  }
  char buffer[9000];
  int j;
  for (int i=0; i < 9000; i++) {
    j = i % 26;
      buffer[i] = 'A'+j;
  }
  buffer[8999] = 0;
  unsigned long numwrite = 9000;
  res = fs_write(handle, (char*) &buffer, &numwrite);
  if (res < 0) {
    printf("ERROR: %d\n", res);
  }
  fs_close(handle);
  printf("\n%lu bytes written.", numwrite);
}

void delit(char* name)
{
  int res;
  res = fs_unlink(name, 1);
  if (res < 0) {
    printf("ERROR: %d\n", res);
  }
}

void fsmkdir(char* name)
{
  int res;
  res = fs_mkdir(name, 1);
  if (res < 0) {
    printf("ERROR: %d\n", res);
  }
}

void fsrmdir(char* name)
{
  int res;
  res = fs_rmdir(name, 1);
  if (res < 0) {
    printf("ERROR: %d\n", res);
  }
}



int main(int argc, char* argv[])
{
  // get console focus      [from checkbios.cpp]
  // (and remember to give it back :)
  //
  getFocus();
  atexit(releaseFocus);

  if (argc > 2) {
    if (strcmp(argv[1], "dir") == 0)
      dirl("C:\\Alizée\\*.*" /*argv[2]*/);
    if (strcmp(argv[1], "dirs") == 0)
      dirs(argv[2]);
    else if (strcmp(argv[1],"write") == 0)
      write(argv[2]);
    else if (strcmp(argv[1],"type") == 0)
      typeit(argv[2]);
    else if (strcmp(argv[1],"md") == 0)
      fsmkdir(argv[2]);
    else if (strcmp(argv[1],"rd") == 0)
      fsrmdir(argv[2]);
    else if (strcmp(argv[1],"del") == 0)
      delit(argv[2]);
  } else if (argc == 2) {
    if (strcmp(argv[1], "dir") == 0)
      dirl("*.*");
    else if (strcmp(argv[1], "dirs") == 0)
      dirs("*.*");
    else if (strcmp(argv[1],"unmount") == 0)
      drv_unmount();
  } else
    printf("At least 1 Parameter has to be passed!\n");
  return 0;
}
